indexing
	description: "External %"C%" functions for GMP_INTEGER_REFs"
	author: "Christopher John Saunders"
	date: "$Date: 2003/10/17 $"
	revision: "$Revision: 1.0 $"

class
	GMP_INTEGER_FUNCTIONS

feature -- Initialization Functions

	mpz_init (p: POINTER) is
			-- Initialize `p', and set its value to 0.
		external
			"C (mpz_ptr) | %"gmp.h%""
		end

	mpz_clear (p: POINTER) is
			-- Free the space occupied by `p'.
		external
			"C (mpz_ptr) | %"gmp.h%""
		end

feature -- Assignment Functions

	mpz_set (rop, op: POINTER) is
			-- Set the value of `rop' from `op'.
		external
			"C (mpz_ptr, mpz_srcptr) | %"gmp.h%""
		end

	mpz_set_si (rop: POINTER; op: INTEGER) is
			-- Set the value of `rop' from `op'.
		external
			"C (mpz_ptr, signed long int) | %"gmp.h%""
		end

	mpz_set_d (rop: POINTER; op: DOUBLE) is
			-- Set the value of `rop' from `op'.
		external
			"C (mpz_ptr, double) | %"gmp.h%""
		end

	mpz_set_q (rop, op: POINTER) is
			-- Set the value of `rop' from `op'.
		external
			"C (mpz_ptr, mpq_srcptr) | %"gmp.h%""
		end

	mpz_set_f (rop, op: POINTER) is
			-- Set the value of `rop' from `op'.
		external
			"C (mpz_ptr, mpf_srcptr) | %"gmp.h%""
		end

	mpz_set_str (rop, str: POINTER; a_base: INTEGER) is
			-- Set the value of `rop' from `str' in base `a_base'.
		external
			"C (mpz_ptr, __gmp_const char *, int) | %"gmp.h%""
		end

feature -- Combined Initialization and Assignment Functions

	mpz_init_set (rop, op: POINTER) is
			-- Initialize and set the initial numeric value from `op'.
		external
			"C (mpz_ptr, mpz_srcptr) | %"gmp.h%""
		end

	mpz_init_set_si (rop: POINTER op: INTEGER) is
			-- Initialize and set the initial numeric value from `op'.
		external
			"C (mpz_ptr, signed long int) | %"gmp.h%""
		end

	mpz_init_set_d (rop: POINTER; op: DOUBLE) is
			-- Initialize and set the initial numeric value from `op'.
		external
			"C (mpz_ptr, double) | %"gmp.h%""
		end

	mpz_init_set_str (rop, str: POINTER; a_base: INTEGER) is
			-- Initialize and set the initial numeric value from `str' in base `a_base'.
		external
			"C (mpz_ptr, __gmp_const char *, int) | %"gmp.h%""
		end

feature -- Conversion Functions

	mpz_get_si (op: POINTER): INTEGER is
			-- If `op' fits into a signed long int return the value of op.
			-- Otherwise return the least significant part of `op', with the same sign as `op'.
		external
			"C (mpz_srcptr): EIF_INTEGER | %"gmp.h%""
		end

	mpz_get_d (op: POINTER): DOUBLE is
			-- Convert op to a double.
		external
			"C (mpz_srcptr): EIF_DOUBLE | %"gmp.h%""
		end

	mpz_get_d_2exp (exp, op: POINTER): DOUBLE is
			-- Find `Result' and `exp' such that `Result' times 2 raised to `exp',
			-- with 0.5 <= (`Result').abs < 1, is a good approximation to `op'. 
		external
			"C (signed long int *, mpz_srcptr): EIF_DOUBLE | %"gmp.h%""
		end

	mpz_get_str (str: POINTER; a_base: INTEGER; op: POINTER) is
			-- Convert `op' to a string of digits in base `a_base'.
			-- `a_base' may vary from 2 to 36.
		external
			"C (char *, int, mpz_srcptr) | %"gmp.h%""
		end

feature -- Arithmetic Functions

	mpz_add (rop, op1, op2: POINTER) is
			-- Set `rop' to `op1' + `op2'.
		external
			"C (mpz_ptr, mpz_srcptr, mpz_srcptr) | %"gmp.h%""
		end

	mpz_sub (rop, op1, op2: POINTER) is
			-- Set `rop' to `op1' - `op2'.
		external
			"C (mpz_ptr, mpz_srcptr, mpz_srcptr) | %"gmp.h%""
		end

	mpz_mul (rop, op1, op2: POINTER) is
			-- Set `rop' to `op1' * `op2'.
		external
			"C (mpz_ptr, mpz_srcptr, mpz_srcptr) | %"gmp.h%""
		end

	mpz_mul_2exp (rop, op1: POINTER; op2: INTEGER) is
			-- Set `rop' to `op1' times 2 raised to `op2'.
			-- This operation can also be defined as a left shift by `op2' bits.
		external
			"C (mpz_ptr, mpz_srcptr, unsigned long int) | %"gmp.h%""
		end

	mpz_neg (rop, op: POINTER) is
			-- Set `rop' to -`op'.
		external
			"C (mpz_ptr, mpz_srcptr) | %"gmp.h%""
		end

	mpz_abs (rop, op: POINTER) is
			-- Set `rop' to the absolute value of `op'.
		external
			"C (mpz_ptr, mpz_srcptr) | %"gmp.h%""
		end

feature -- Division Functions

	mpz_tdiv_q (q, n, d: POINTER) is
			-- Divide `n' by `d', forming a quotient `q'.
			-- Round towards 0.
		external
			"C (mpz_ptr, mpz_srcptr, mpz_srcptr) | %"gmp.h%""
		end

	mpz_tdiv_r (r, n, d: POINTER) is
			-- Divide `n' by `d', forming a remainder `r'.
			-- Round towards 0.
		external
			"C (mpz_ptr, mpz_srcptr, mpz_srcptr) | %"gmp.h%""
		end

	mpz_divisible_p (n, d: POINTER): INTEGER is
			-- Non-zero if `n' is exactly divisible by `d',.
		external
			"C (mpz_srcptr, mpz_srcptr): EIF_INTEGER | %"gmp.h%""
		end

	mpz_congruent_p (n, c, d: POINTER): INTEGER is
			-- Non-zero if `n' is congruent to `c' modulo `d',.
		external
			"C (mpz_srcptr, mpz_srcptr, mpz_srcptr): EIF_INTEGER | %"gmp.h%""
		end

feature -- Exponentiation Functions

	mpz_pow_ui (rop, a_base: POINTER; exp: INTEGER) is
			-- Divide `n' by `d', forming a quotient `q'.
			-- Round towards 0.
		external
			"C (mpz_ptr, mpz_srcptr, unsigned long int) | %"gmp.h%""
		end

	mpz_ui_pow_ui (rop: POINTER; a_base, exp: INTEGER) is
			-- Divide `n' by `d', forming a quotient `q'.
			-- Round towards 0.
		external
			"C (mpz_ptr, unsigned long int, unsigned long int) | %"gmp.h%""
		end

feature -- Root Extraction Functions

	mpz_root (rop, op: POINTER; n: INTEGER): INTEGER is
			-- Set `rop' to the truncated integer part of the nth root of `op'.
		external
			"C (mpz_ptr, mpz_srcptr, unsigned long int): EIF_INTEGER | %"gmp.h%""
		end

	mpz_sqrt (rop, op: POINTER) is
			-- Set `rop' to the truncated integer part of the square root of `op'.
		external
			"C (mpz_ptr, mpz_srcptr) | %"gmp.h%""
		end

	mpz_sqrtrem (rop1, rop2, op: POINTER) is
			-- Set `rop1' to the truncated integer part of the square root of `op', like `mpz_sqrt'.
			-- Set `rop2' to the remainder `op' - `rop1' * `rop1', which will be zero if `op' is a perfect square.
		require
			rop1_not_equal_to_rop2: not (rop1 = rop2)
		external
			"C (mpz_ptr, mpz_ptr, mpz_srcptr) | %"gmp.h%""
		end

	mpz_perfect_power_p (op: POINTER): INTEGER is
			-- Non-zero if `op' is a perfect power.
		external
			"C (mpz_srcptr): EIF_INTEGER | %"gmp.h%""
		end

	mpz_perfect_square_p (op: POINTER): INTEGER is
			-- Non-zero if `op' is a perfect square.
		external
			"C (mpz_srcptr): EIF_INTEGER | %"gmp.h%""
		end

feature -- Number Theoretic Functions

	mpz_probab_prime_p (n: POINTER; reps: INTEGER): INTEGER is
			-- 2 if `n' is definitely prime,
			-- 1 if `n' is probably prime (without being certain),
			-- 0 if `n' is definitely composite.
		external
			"C (mpz_srcptr, int): EIF_INTEGER | %"gmp.h%""
		end

	mpz_nextprime (rop, op: POINTER) is
			-- Set `rop' to the next prime greater than `op'.
		external
			"C (mpz_ptr, mpz_srcptr) | %"gmp.h%""
		end

	mpz_gcd (rop, op1, op2: POINTER) is
			-- Set `rop' to the greatest common divisor of `op1' and `op2'.
			-- `rop' is always positive.
		external
			"C (mpz_ptr, mpz_srcptr, mpz_srcptr) | %"gmp.h%""
		end

	mpz_gcdext (g, s, t, a, b: POINTER) is
			-- Set `g' to the greatest common divisor of `a' and `b',
			-- and in addition set `s' and `t' to coefficients satisfying
			-- `a' * `s' + `b' * `t' = `g'. `g' is always positive,
			-- even if one or both of `a' and `b' are negative.
		external
			"C (mpz_ptr, mpz_ptr, mpz_ptr, mpz_srcptr, mpz_srcptr) | %"gmp.h%""
		end

	mpz_lcm (rop, op1, op2: POINTER) is
			-- Set `rop' to the least common multiple  of `op1' and `op2'.
			-- `rop' will be zero if either `op1' or `op2' is zero.
			-- `rop' is always positive, irrespective of the signs of `op1' and `op2'.
		external
			"C (mpz_ptr, mpz_srcptr, mpz_srcptr) | %"gmp.h%""
		end

	mpz_invert (rop, op1, op2: POINTER): INTEGER is
			-- Compute the inverse of `op1' modulo op2 and put the result in `rop'.
			-- If the inverse exists, the return value is non-zero and `rop' will
			-- satisfy 0 <= `rop' < `op2'. If an inverse doesn't exist the return
			-- value is 0 and `rop' is undefined.
		external
			"C (mpz_ptr, mpz_srcptr, mpz_srcptr): EIF_INTEGER | %"gmp.h%""
		end

	mpz_jacobi (a, b: POINTER): INTEGER is
			-- Calculate the Jacobi symbol (`a' /`b').
			-- This is defined only for `b' odd.
		external
			"C (mpz_srcptr, mpz_srcptr): EIF_INTEGER | %"gmp.h%""
		end

	mpz_legendre (a, p: POINTER): INTEGER is
			-- Calculate the Legendre symbol (`a' / `p').
			-- This is defined only for `p' an odd positive prime,
			-- and for such `p' it's identical to the Jacobi symbol.
		external
			"C (mpz_srcptr, mpz_srcptr): EIF_INTEGER | %"gmp.h%""
		end

	mpz_kronecker (a, b: POINTER): INTEGER is
			-- Calculate the Jacobi symbol (`a' / `b') with the Kronecker extension
			-- (`a' / 2) = (2 / `a') when `a' odd, or (`a' / 2) = 0 when `a' even.
			-- When `b' is odd the Jacobi symbol and Kronecker symbol are identical.

		external
			"C (mpz_srcptr, mpz_srcptr): EIF_INTEGER | %"gmp.h%""
		end

	mpz_remove (rop, op, f: POINTER): INTEGER is
			-- Remove all occurrences of the factor `f' from `op' and store the result in `rop'.
			-- The return value is how many such occurrences were removed.

		external
			"C (mpz_ptr, mpz_srcptr, mpz_srcptr): EIF_INTEGER | %"gmp.h%""
		end

	mpz_fac_ui (rop: POINTER; op: INTEGER) is
			-- Set `rop' to `op'!, the factorial of `op'.
		external
			"C (mpz_ptr, unsigned long int) | %"gmp.h%""
		end

	mpz_bin_ui (rop, n: POINTER; k: INTEGER) is
			-- Compute the binomial coefficient `n' over `k' and store the result in `rop'.
			-- Negative values of `n' are supported using the identity
			-- bin(-`n', `k') = (-1) ^ `k' * bin(`n' + `k' - 1, `k'),
		external
			"C (mpz_ptr, mpz_srcptr, unsigned long int) | %"gmp.h%""
		end

	mpz_bin_uiui (rop: POINTER; n, k: INTEGER) is
			-- Compute the binomial coefficient `n' over `k' and store the result in `rop'.
		external
			"C (mpz_ptr, unsigned long int, unsigned long int) | %"gmp.h%""
		end

	mpz_fib_ui (fn: POINTER; n: INTEGER) is
			-- Set `fn' to to F[n], the n'th Fibonacci number.
		external
			"C (mpz_ptr, unsigned long int) | %"gmp.h%""
		end

	mpz_fib2_ui (fn, fnsub1: POINTER; n: INTEGER) is
			-- Set `fn' to F[n], and `fnsub1' to F[n-1].
		external
			"C (mpz_ptr, mpz_ptr, unsigned long int) | %"gmp.h%""
		end

	mpz_lucnum_ui (ln: POINTER; n: INTEGER) is
			-- Set `ln' to to L[n], the n'th Lucas number.
		external
			"C (mpz_ptr, unsigned long int) | %"gmp.h%""
		end

	mpz_lucnum2_ui (ln, lnsub1: POINTER; n: INTEGER) is
			-- Set `ln' to L[n], and `lnsub1' to L[n-1].
		external
			"C (mpz_ptr, mpz_ptr, unsigned long int) | %"gmp.h%""
		end

feature -- Comparison Functions

	mpz_cmp (op1, op2: POINTER): INTEGER is
			-- Return a positive value if `op1' > `op2',
			-- 0 if `op1' = `op2', or a negative value
			-- if `op1' < `op2'.
		external
			"C (mpz_srcptr, mpz_srcptr): EIF_INTEGER | %"gmp.h%""
		end

	mpz_cmpabs (op1, op2: POINTER): INTEGER is
			-- Return a positive value if `op1'.abs > `op2'.abs,
			-- 0 if `op1'.abs = `op2'.abs, or a negative value
			-- if `op1'.abs < `op2'.abs.
		external
			"C (mpz_srcptr, mpz_srcptr): EIF_INTEGER | %"gmp.h%""
		end

	mpz_sgn (op: POINTER): INTEGER is
			-- Return +1 if `op' > 0,
			-- 0 if `op' = 0, and
			-- -1 if `op' < 0
		external
			"C [macro %"gmp.h%"] (mpz_srcptr): EIF_INTEGER"
		end

feature -- Logical and Bit Manipulation Functions

	mpz_and (rop, op1, op2: POINTER) is
			-- Set `rop' to `op1' logical-and `op2'.
		external
			"C (mpz_ptr, mpz_srcptr, mpz_srcptr) | %"gmp.h%""
		end

	mpz_ior (rop, op1, op2: POINTER) is
			-- Set `rop' to `op1' inclusive-or `op2'.
		external
			"C (mpz_ptr, mpz_srcptr, mpz_srcptr) | %"gmp.h%""
		end

	mpz_xor (rop, op1, op2: POINTER) is
			-- Set `rop' to `op1' exclusive-or `op2'.
		external
			"C (mpz_ptr, mpz_srcptr, mpz_srcptr) | %"gmp.h%""
		end

	mpz_com (rop, op: POINTER) is
			-- Set `rop' to the one's complement of `op'.
		external
			"C (mpz_ptr, mpz_srcptr) | %"gmp.h%""
		end

	mpz_popcount (op: POINTER): INTEGER is
			-- If `op' >= 0, return the population count of `op',
			-- which is the number of 1 bits in the binary representation.
			-- If `op' < 0, the number of 1s is infinite, and the return value is MAX_ULONG,
			-- the largest possible unsigned long.
		external
			"C (mpz_srcptr): EIF_INTEGER | %"gmp.h%""
		end

	mpz_hamdist (op1, op2: POINTER): INTEGER is
			-- If `op1' and `op2' are both >=0 or both <0, return the hamming distance
			-- between the two operands, which is the number of bit positions where `op1'
			-- and `op2' have different bit values.
			-- If one operand is >=0 and the other <0 then the number of bits different
			-- is infinite, and the return value is MAX_ULONG, the largest possible unsigned long.
		external
			"C (mpz_srcptr, mpz_srcptr): EIF_INTEGER | %"gmp.h%""
		end

	mpz_scan0 (op: POINTER; starting_bit: INTEGER): INTEGER is
			-- Scan `op', starting from bit `starting_bit', towards more significant bits,
			-- until the first 0 bit is found.
			-- Return the index of the found bit.
		external
			"C (mpz_srcptr, unsigned long int): EIF_INTEGER | %"gmp.h%""
		end

	mpz_scan1 (op: POINTER; starting_bit: INTEGER): INTEGER is
			-- Scan `op', starting from bit `starting_bit', towards more significant bits,
			-- until the first 1 bit is found.
			-- Return the index of the found bit.
		external
			"C (mpz_srcptr, unsigned long int): EIF_INTEGER | %"gmp.h%""
		end

	mpz_setbit (rop: POINTER; bit_index: INTEGER) is
			-- Set bit `bit_index' in `rop'.
		external
			"C (mpz_ptr, unsigned long int) | %"gmp.h%""
		end

	mpz_clrbit (rop: POINTER; bit_index: INTEGER) is
			-- Clear bit `bit_index' in `rop'.
		external
			"C (mpz_ptr, unsigned long int) | %"gmp.h%""
		end

	mpz_tstbit (op: POINTER; bit_index: INTEGER): INTEGER is
			-- Test bit `bit_index' in `op' and return 0 or 1 accordingly.
		external
			"C (mpz_srcptr, unsigned long int): EIF_INTEGER | %"gmp.h%""
		end

feature -- Miscellaneous Functions

	mpz_fits_ulong_p (op: POINTER): INTEGER is
			-- Return non-zero iff the value of `op' fits in an unsigned long int.
		external
			"C (mpz_srcptr): EIF_INTEGER | %"gmp.h%""
		end

	mpz_fits_slong_p (op: POINTER): INTEGER is
			-- Return non-zero iff the value of `op' fits in a signed long int.
		external
			"C (mpz_srcptr): EIF_INTEGER | %"gmp.h%""
		end

	mpz_fits_uint_p (op: POINTER): INTEGER is
			-- Return non-zero iff the value of `op' fits in an unsigned int.
		external
			"C (mpz_srcptr): EIF_INTEGER | %"gmp.h%""
		end

	mpz_fits_sint_p (op: POINTER): INTEGER is
			-- Return non-zero iff the value of `op' fits in a signed int.
		external
			"C (mpz_srcptr): EIF_INTEGER | %"gmp.h%""
		end

	mpz_fits_ushort_p (op: POINTER): INTEGER is
			-- Return non-zero iff the value of `op' fits in a unsigned short int.
		external
			"C (mpz_srcptr): EIF_INTEGER | %"gmp.h%""
		end

	mpz_fits_sshort_p (op: POINTER): INTEGER is
			-- Return non-zero iff the value of `op' fits in a signed short int.
		external
			"C (mpz_srcptr): EIF_INTEGER | %"gmp.h%""
		end

	mpz_odd_p (op: POINTER): INTEGER is
			-- Determine whether `op' is odd.
			-- Return non-zero if yes, zero if no. 
		external
			"C [macro %"gmp.h%"] (mpz_srcptr): EIF_INTEGER"
		end

	mpz_even_p (op: POINTER): INTEGER is
			-- Determine whether op is even.
			-- Return non-zero if yes, zero if no. 
		external
			"C [macro %"gmp.h%"] (mpz_srcptr): EIF_INTEGER"
		end

	mpz_sizeinbase (op: POINTER; a_base: INTEGER): INTEGER is
			-- Return the size of `op' measured in number of digits in base `a_base'.
			-- The base may vary from 2 to 36.
			-- The sign of `op' is ignored, just the absolute value is used.
			-- The result will be exact or 1 too big.
			-- If `a_base' is a power of 2, the result will always be exact.
			-- If `op' is zero the return value is always 1.
			-- This function is useful in order to allocate the right amount of
			-- space before converting op to a string.
			-- The right amount of allocation is normally two more than the value
			-- returned (one extra for a minus sign and one for the null-terminator).
 

		external
			"C (mpz_srcptr, int): EIF_INTEGER | %"gmp.h%""
		end

end -- class GMP_INTEGER_FUNCTIONS
