indexing
	description: "External %"C%" functions for GMP_RATIONAL_REFs"
	author: "Christopher John Saunders"
	date: "$Date: 2003/10/17 $"
	revision: "$Revision: 1.0 $"

class
	GMP_RATIONAL_FUNCTIONS

feature -- Initialization and Assignment Functions

	mpq_init (p: POINTER) is
			-- Initialize `p', and set its value to 0 / 1.
		external
			"C (mpq_ptr) | %"gmp.h%""
		end

	mpq_clear (p: POINTER) is
			-- Free the space occupied by `p'.
		external
			"C (mpq_ptr) | %"gmp.h%""
		end

	mpq_set (rop, op: POINTER) is
			-- Assign `rop' from `op'.
		external
			"C (mpq_ptr, mpq_srcptr) | %"gmp.h%""
		end

	mpq_set_z (rop, op: POINTER) is
			-- Assign `rop' from `op'.
		external
			"C (mpq_ptr, mpz_srcptr) | %"gmp.h%""
		end

	mpq_set_si (rop: POINTER; op1, op2: INTEGER) is
			-- Assign `rop' to `op1' / `op1'.
		external
			"C (mpq_ptr, signed long int, unsigned long int) | %"gmp.h%""
		end

	mpq_set_str (rop, str2: POINTER; a_base: INTEGER) is
			-- Set `rop' from a null-terminated string `str' in the given base `a_base'.
		external
			"C (mpq_ptr, const char *, int) | %"gmp.h%""
		end

	mpq_canonicalize (rop: POINTER) is
			-- 
		external
			"C (mpq_ptr) | %"gmp.h%""
		end

feature -- Conversion Functions

	mpq_get_d (op: POINTER): DOUBLE is
			-- Convert `op' to a double.
		external
			"C (mpq_srcptr): EIF_DOUBLE | %"gmp.h%""
		end

	mpq_set_d (rop: POINTER; op: DOUBLE) is
			-- Set `rop' to the value of `op', without rounding.
		external
			"C (mpq_ptr, double) | %"gmp.h%""
		end

	mpq_set_f (rop, op: POINTER) is
			-- Set `rop' to the value of `op', without rounding.
		external
			"C (mpq_ptr, mpf_srcptr) | %"gmp.h%""
		end

	mpq_get_str (str: POINTER; a_base: INTEGER; op: POINTER) is
			-- Convert `op' to a string of digits in base `a_base'.
			-- If `str' is not `default_pointer', it should point to a block of storage large enough for the result, that being
			-- mpz_sizeinbase (mpq_numref(op), base) + mpz_sizeinbase (mpq_denref(op), base) + 3

		external
			"C (char *, int, mpq_srcptr) | %"gmp.h%""
		end

feature -- Arithmetic Functions

	mpq_add (sum, addend1, addend2: POINTER) is
			-- Set `sum' to `addend1' + `addend2'.
		external
			"C (mpq_ptr, mpq_srcptr, mpq_srcptr) | %"gmp.h%""
		end

	mpq_sub (difference, minuend, subtrahend: POINTER) is
			-- Set `difference' to `minuend' - `subtrahend'.
		external
			"C (mpq_ptr, mpq_srcptr, mpq_srcptr) | %"gmp.h%""
		end

	mpq_mul (product, multiplier, multiplicand: POINTER) is
			-- Set `product' to `multiplier' times `multiplicand'.
		external
			"C (mpq_ptr, mpq_srcptr, mpq_srcptr) | %"gmp.h%""
		end

	mpq_mul_2exp (rop, op1: POINTER; op2: INTEGER) is
			-- Set `rop' to `op1' times 2 raised to `op2'.
		external
			"C (mpq_ptr, mpq_srcptr, unsigned long) | %"gmp.h%""
		end

	mpq_div (quotient, dividend, divisor: POINTER) is
			-- Set `quotient' to `dividend' / `divisor'.
		external
			"C (mpq_ptr, mpq_srcptr, mpq_srcptr) | %"gmp.h%""
		end

	mpq_div_2exp (rop, op1: POINTER; op2: INTEGER) is
			-- Set rop to op1 divided by 2 raised to op2.
		external
			"C (mpq_ptr, mpq_srcptr, unsigned long) | %"gmp.h%""
		end

	mpq_neg (negated_operand, operand: POINTER) is
			-- Set `negated_operand' to -`operand'.
		external
			"C (mpq_ptr, mpq_srcptr) | %"gmp.h%""
		end

	mpq_abs (rop, op: POINTER) is
			-- Set `rop' to the absolute value of `op'.
		external
			"C (mpq_ptr, mpq_srcptr) | %"gmp.h%""
		end

	mpq_inv (inverted_number, number: POINTER) is
			-- Set `inverted_number' to 1 / `number'.
			-- If the new denominator is zero, this routine will divide by zero.
		external
			"C (mpq_ptr, mpq_srcptr) | %"gmp.h%""
		end

feature -- Comparison Functions

	mpq_cmp (op1, op2: POINTER): INTEGER is
			-- Return a positive value if `op1' > `op2',
			-- zero if `op1' = `op2', and a negative value if `op1' < `op2'.
		external
			"C (mpq_srcptr, mpq_srcptr): EIF_INTEGER | %"gmp.h%""
		end

	mpq_sgn (op: POINTER): INTEGER is
			-- Return +1 if `op' > 0, 0 if `op' = 0, and -1 if `op' < 0.
		external
			"C [macro %"gmp.h%"] (mpq_srcptr): EIF_INTEGER"
		end

	mpq_equal (op1, op2: POINTER): INTEGER is
			-- Return non-zero if `op1' and `op2' are equal, zero if they are non-equal.
		external
			"C (mpq_srcptr, mpq_srcptr): EIF_INTEGER | %"gmp.h%""
		end

feature -- Applying Integer Functions to Rationals

	mpq_get_num (a_numerator, a_rational: POINTER) is
			-- Set `a_numerator' to the numerator of `a_rational'.
		external
			"C (mpz_ptr, mpq_srcptr) | %"gmp.h%""
		end

	mpq_get_den (a_denominator, a_rational: POINTER) is
			-- Set `a_denominator' to the denominator of `a_rational'.
		external
			"C (mpz_ptr, mpq_srcptr) | %"gmp.h%""
		end

	mpq_set_num (a_rational, a_numerator: POINTER) is
			-- Set the numerator of `a_rational' to `a_numerator'.
		external
			"C (mpq_ptr, mpz_srcptr) | %"gmp.h%""
		end

	mpq_set_den (a_rational, a_denominator: POINTER) is
			-- Set the denominator of `a_rational' to `a_denominator'.
		external
			"C (mpq_ptr, mpz_srcptr) | %"gmp.h%""
		end

feature -- Element change

end -- class GMP_RATIONAL_FUNCTIONS
