indexing
	description: "External %"C%" functions for GMP_REAL_REFs"
	author: "Christopher John Saunders"
	date: "$Date: 2003/10/17 $"
	revision: "$Revision: 1.0 $"

class
	GMP_REAL_FUNCTIONS

feature -- Initialization Functions

	mpf_set_default_prec (prec: INTEGER) is
			-- Set the default precision to be at least `prec' bits.
			-- All subsequent calls to mpf_init will use this precision,
			-- but previously initialized variables are unaffected.
		external
			"C (unsigned long int) | %"gmp.h%""
		end

	mpf_get_default_prec: INTEGER is
			-- Return the default default precision actually used.
		external
			"C: EIF_INTEGER | %"gmp.h%""
		end

	mpf_init (p: POINTER) is
			-- Initialize `p', and set its value to 0.
		external
			"C (mpf_ptr) | %"gmp.h%""
		end

	mpf_init2 (p: POINTER; prec: INTEGER) is
			-- Initialize `p' to 0 and set its precision to be at least `prec' bits.
		external
			"C (mpf_ptr, unsigned long int) | %"gmp.h%""
		end

	mpf_clear (p: POINTER) is
			-- Free the space occupied by `p'.
		external
			"C (mpf_ptr) | %"gmp.h%""
		end

	mpf_get_prec (p: POINTER): INTEGER is
			-- Return the current precision of `p', in bits.
		external
			"C (mpf_srcptr): EIF_INTEGER | %"gmp.h%""
		end

	mpf_set_prec (p: POINTER; prec: INTEGER) is
			-- Set the precision of `p' to be at least `prec' bits.
			-- The value in `p' will be truncated to the new precision.
		external
			"C (mpf_ptr, unsigned long int) | %"gmp.h%""
		end

feature -- Assignment Functions

	mpf_set (rop, op: POINTER) is
			-- Set the value of `rop' from `op'.
		external
			"C (mpf_ptr, mpf_srcptr) | %"gmp.h%""
		end

	mpf_set_si (rop: POINTER; op: INTEGER) is
			-- Set the value of `rop' from `op'.
		external
			"C (mpf_ptr, signed long int) | %"gmp.h%""
		end

	mpf_set_d (rop: POINTER; op: DOUBLE) is
			-- Set the value of `rop' from `op'.
		external
			"C (mpf_ptr, double) | %"gmp.h%""
		end

	mpf_set_z (rop, op: POINTER) is
			-- Set the value of `rop' from `op'.
		external
			"C (mpf_ptr, mpz_srcptr) | %"gmp.h%""
		end

	mpf_set_q (rop, op: POINTER) is
			-- Set the value of `rop' from `op'.
		external
			"C (mpf_ptr, mpq_srcptr) | %"gmp.h%""
		end

	mpf_set_str (rop, str: POINTER; base: INTEGER) is
			-- Set the value of `rop' from the string in `str'.
		external
			"C (mpf_ptr, __gmp_const char *, int) | %"gmp.h%""
		end

feature -- Combined Initialization and Assignment Functions

	mpf_init_set (rop, op: POINTER) is
			-- Set the value of `rop' from `op'.
		external
			"C (mpf_ptr, mpf_srcptr) | %"gmp.h%""
		end

	mpf_init_set_si (rop: POINTER; op: INTEGER) is
			-- Set the value of `rop' from `op'.
		external
			"C (mpf_ptr, signed long int) | %"gmp.h%""
		end

	mpf_init_set_d (rop: POINTER; op: DOUBLE) is
			-- Set the value of `rop' from `op'.
		external
			"C (mpf_ptr, double) | %"gmp.h%""
		end

	mpf_init_set_str (rop, str: POINTER; base: INTEGER) is
			-- Set the value of `rop' from the string in `str'.
		external
			"C (mpf_ptr, __gmp_const char *, int) | %"gmp.h%""
		end

feature -- Conversion Functions

	mpf_get_d (op: POINTER): DOUBLE is
			-- Convert `op' to a double.
		external
			"C (mpf_srcptr): EIF_DOUBLE | %"gmp.h%""
		end

	mpf_get_d_2exp (exp, op: POINTER): DOUBLE is
			-- Find `Result' and `exp' such that `Result' times 2 raised to `exp',
			-- with 0.5 <= `Result'.abs < 1, is a good approximation to `op'.
		external
			"C (signed long int *, mpf_srcptr): EIF_DOUBLE | %"gmp.h%""
		end

	mpf_get_si (op: POINTER): INTEGER is
			-- Find `Result' and `exp' such that `Result' times 2 raised to `exp',
			-- with 0.5 <= `Result'.abs < 1, is a good approximation to `op'.
		external
			"C (mpf_srcptr): EIF_INTEGER | %"gmp.h%""
		end

	mpf_get_str (str, expptr: POINTER;  a_base, n_digits: INTEGER; op: POINTER): POINTER is
			-- Convert `op' to a string of digits in base `a_base'.
			-- `a_base' can be 2 to 36.
			-- Up to `n_digits' digits will be generated.
			-- Trailing zeros are not returned.
			-- No more digits than can be accurately represented by `op' are ever generated.
			-- If `n_digits' is 0 then that accurate maximum number of digits are generated.
		external
			"C (char *, mp_exp_t *, int, size_t, mpf_srcptr): EIF_INTEGER | %"gmp.h%""
		end

feature -- Arithmetic Functions

	mpf_add (rop, op1, op2: POINTER) is
			-- Set `rop' to `op1' + `op2'.
		external
			"C (mpf_ptr, mpf_srcptr, mpf_srcptr) | %"gmp.h%""
		end

	mpf_sub (rop, op1, op2: POINTER) is
			-- Set `rop' to `op1' - `op2'.
		external
			"C (mpf_ptr, mpf_srcptr, mpf_srcptr) | %"gmp.h%""
		end

	mpf_mul (rop, op1, op2: POINTER) is
			-- Set `rop' to `op1' * `op2'.
		external
			"C (mpf_ptr, mpf_srcptr, mpf_srcptr) | %"gmp.h%""
		end

	mpf_div (rop, op1, op2: POINTER) is
			-- Set `rop' to `op1' / `op2'.
		external
			"C (mpf_ptr, mpf_srcptr, mpf_srcptr) | %"gmp.h%""
		end

	mpf_sqrt (rop, op: POINTER) is
			-- Set `rop' to the square root of `op'.
		external
			"C (mpf_ptr, mpf_srcptr) | %"gmp.h%""
		end

	mpf_sqrt_ui (rop: POINTER; op: INTEGER) is
			-- Set `rop' to the square root of `op'.
		external
			"C (mpf_ptr, unsigned long int) | %"gmp.h%""
		end

	mpf_pow_ui (rop, op1: POINTER; op2: INTEGER) is
			-- Set `rop' to `op1' raised to the power `op2'.
		external
			"C (mpf_ptr, mpf_srcptr, unsigned long int) | %"gmp.h%""
		end

	mpf_neg (rop, op: POINTER) is
			-- Set `rop' to -`op'.
		external
			"C (mpf_ptr, mpf_srcptr) | %"gmp.h%""
		end

	mpf_abs (rop, op: POINTER) is
			-- Set `rop' to the absolute value of `op'.
		external
			"C (mpf_ptr, mpf_srcptr) | %"gmp.h%""
		end

	mpf_mul_2exp (rop, op1: POINTER; op2: INTEGER) is
			-- Set `rop' to `op1' times 2 raised to `op2'.
		external
			"C (mpf_ptr, mpf_srcptr, unsigned long int) | %"gmp.h%""
		end

	mpf_div_2exp (rop, op1: POINTER; op2: INTEGER) is
			-- Set `rop' to `op1' divided by 2 raised to `op2'.
		external
			"C (mpf_ptr, mpf_srcptr, unsigned long int) | %"gmp.h%""
		end

feature -- Comparison Functions

	mpf_cmp (op1, op2: POINTER): INTEGER is
			-- Return a positive value if `op1' > `op2',
			-- zero if `op1' = `op2', and a negative value if `op1' < `op2'.
		external
			"C (mpf_srcptr, mpf_srcptr): EIF_INTEGER | %"gmp.h%""
		end

	mpf_reldiff (rop, op1, op2: POINTER) is
			-- Set `rop' to `op1' + `op2'.
		external
			"C (mpf_ptr, mpf_srcptr, mpf_srcptr) | %"gmp.h%""
		end

	mpf_sgn (op: POINTER): INTEGER is
			-- Return +1 if `op' > 0,
			-- 0 if `op' = 0, and
			-- -1 if `op' < 0
		external
			"C [macro %"gmp.h%"] (mpf_srcptr): EIF_INTEGER"
		end

feature -- Miscellaneous Functions

	mpf_ceil (rop, op: POINTER) is
			-- Round to the next higher integer,
		external
			"C (mpf_ptr, mpf_srcptr) | %"gmp.h%""
		end

	mpf_floor (rop, op: POINTER) is
			-- Round to the next lower integer,
		external
			"C (mpf_ptr, mpf_srcptr) | %"gmp.h%""
		end

	mpf_trunc (rop, op: POINTER) is
			-- Truncate to an integer,
		external
			"C (mpf_ptr, mpf_srcptr) | %"gmp.h%""
		end

	mpf_integer_p (op: POINTER): INTEGER is
			-- Return non-zero if `op' is an integer.
		external
			"C (mpf_srcptr): EIF_INTEGER | %"gmp.h%""
		end

	mpf_fits_slong_p (op: POINTER): INTEGER is
			-- Return non-zero if `op' fits a "C" signed
			-- long int when truncated.
		external
			"C (mpf_srcptr): EIF_INTEGER | %"gmp.h%""
		end

	mpf_fits_sshort_p (op: POINTER): INTEGER is
			-- Return non-zero if `op' fits a "C" signed
			-- short int when truncated.
		external
			"C (mpf_srcptr): EIF_INTEGER | %"gmp.h%""
		end

end -- class GMP_REAL_FUNCTIONS
