indexing
	description: "External %"C%" functions for MPFR_REFs"
	author: "Christopher John Saunders"
	date: "$Date: 2003/10/17 $"
	revision: "$Revision: 1.0 $"

class
	MPFR_FUNCTIONS

feature -- Rounding modes

	mpfr_set_default_rounding_mode (rnd: INTEGER) is
			-- Set the default rounding mode to `rnd'
		external
			"C (mp_rnd_t) | %"mpfr.h%""
		end

	mpfr_round_prec (x: POINTER; rnd, prec: INTEGER): INTEGER is
			-- Round `x' in rounding mode `rnd' with precision `prec'
		external
			"C (mpfr_ptr, mp_rnd_t, mp_prec_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_print_rnd_mode (rnd: INTEGER): POINTER is
			-- "C" string representing the rounding mode `rnd'
			-- If `rnd' is not a valid mode then `default_pointer'
		external
			"C (mp_rnd_t): EIF_POINTER | %"mpfr.h%""
		end

feature -- Initialization and assignment

	mpfr_set_default_prec (prec: INTEGER) is
			-- Set default precision to exactly `prec' bits
		external
			"C (mp_prec_t) | %"mpfr.h%""
		end

	mpfr_get_default_prec: INTEGER is
			-- Default precision in bits
		external
			"C: EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_init (x: POINTER) is
			-- Initialize `x' to NaN
		external
			"C (mpfr_ptr) | %"mpfr.h%""
		end

	mpfr_init2 (x: POINTER; prec: INTEGER) is
			-- Initialize `x' to NaN with precision `prec'
		external
			"C (mpfr_ptr, mp_prec_t) | %"mpfr.h%""
		end

	mpfr_clear (x: POINTER) is
			-- Free memory occupied by `x'
		external
			"C (mpfr_ptr) | %"mpfr.h%""
		end

	mpfr_set_prec (x: POINTER; prec: INTEGER): INTEGER is
			-- Reset the precision of `x' to exactly `prec' bits
			-- Value stored in `x' is lost
		external
			"C (mpfr_ptr, mp_prec_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_get_prec (x: POINTER): INTEGER is
				-- Precision of `x'
		external
			"C (mpfr_srcptr): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_set_prec_raw (x: POINTER; prec: INTEGER) is
			-- Reset the precision of `x' to exactly `prec' bits
			-- Value stored in `x' is lost
			-- `prec' must be small enough so that the mantissa fits
			-- into the currently allocated memory space for `x'
		external
			"C (mpfr_ptr, mp_prec_t) | %"mpfr.h%""
		end

feature -- Assignment

	mpfr_set (rop, op: POINTER; rnd: INTEGER): INTEGER is
			-- Set `rop' from `op' with rounding mode `rnd'
			-- Returns: 0 if `rop' = `op'
			--          positive if `rop' > `op'
			--          negative if `rop' < `op'
		external
			"C [macro %"mpfr.h%"] (mpfr_ptr, mpfr_srcptr, mp_rnd_t): EIF_INTEGER"
		end

	mpfr_set_ui (rop: POINTER; op, rnd: INTEGER): INTEGER is
			-- Set `rop' from `op' with rounding mode `rnd'
			-- Returns: 0 if `rop' = `op'
			--          positive if `rop' > `op'
			--          negative if `rop' < `op'
		external
			"C (mpfr_ptr, unsigned long, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_set_si (rop: POINTER; op, rnd: INTEGER): INTEGER is
			-- Set `rop' from `op' with rounding mode `rnd'
			-- Returns: 0 if `rop' = `op'
			--          positive if `rop' > `op'
			--          negative if `rop' < `op'
		external
			"C (mpfr_ptr, long, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_set_d (rop: POINTER; op: DOUBLE; rnd: INTEGER): INTEGER is
			-- Set `rop' from `op' with rounding mode `rnd'
			-- Returns: 0 if `rop' = `op'
			--          positive if `rop' > `op'
			--          negative if `rop' < `op'
		external
			"C (mpfr_ptr, double, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_set_z (rop, op: POINTER; rnd: INTEGER): INTEGER is
			-- Set `rop' from `op' with rounding mode `rnd'
			-- Returns: 0 if `rop' = `op'
			--          positive if `rop' > `op'
			--          negative if `rop' < `op'
		external
			"C (mpfr_ptr, mpz_srcptr, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_set_q (rop, op: POINTER; rnd: INTEGER): INTEGER is
			-- Set `rop' from `op' with rounding mode `rnd'
			-- Returns: 0 if `rop' = `op'
			--          positive if `rop' > `op'
			--          negative if `rop' < `op'
		external
			"C (mpfr_ptr, mpq_srcptr, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_set_str (rop, s: POINTER; a_base, rnd: INTEGER): INTEGER is
				-- Set `rop' to the value represented by `s' in base `a_base'
				-- with rounding mode `rnd'
				-- Returns: -1 if an internal overflow occurs
				--           0 if no error occured
				--           1 if input is incorrect
		external
			"C (mpfr_ptr, __gmp_const char *, int, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_set_str_raw (rop, s: POINTER) is
			-- Set `rop' to the value represented by `s'
			-- `s' must be of the form +/-xxxx.xxxxxxEyy
			-- The exponent is read in decimal but is interpreted
			-- a the power of 2 to be multiplied by the mantissa
			-- The mantissa length of `s' must be <= the precision of `x'
			-- If `s' starts with "N" then `x' is set to NaN
			-- If `s' starts with "I" after the sign then `x' is set
			-- to infinity with the corresponding sign
		external
			"C (mpfr_ptr, char *t) | %"mpfr.h%""
		end

	mpfr_set_f (rop, op: POINTER; rnd: INTEGER): INTEGER is
			-- Set `rop' from `op' with rounding mode `rnd'
			-- Returns: 0 if `rop' = `op'
			--          positive if `rop' > `op'
			--          negative if `rop' < `op'
		external
			"C (mpfr_ptr, mpf_srcptr, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_set_inf (x: POINTER; a_sign: INTEGER) is
			-- Set `x' to infinity
			-- `x' will be positive iff `a_sign' is positive
		external
			"C (mpfr_ptr, int) | %"mpfr.h%""
		end

	mpfr_set_nan (x: POINTER) is
			-- Set `x' to NaN
		external
			"C (mpfr_ptr) | %"mpfr.h%""
		end

	mpfr_swap (x, y: POINTER) is
			-- Swap `x' and `y'
		external
			"C (mpfr_ptr, mpfr_ptr) | %"mpfr.h%""
		end

feature -- Combined initialization and assignment

	mpfr_init_set (rop, op: POINTER; rnd: INTEGER): INTEGER is
			-- Initialize `rop' to the value `op' rounded in the direction `rnd'
			-- Default precision is used
			-- Returns: 0 if `rop' = `op'
			--          positive if `rop' > `op'
			--          negative if `rop' < `op'
		external
			"C [macro %"mpfr.h%"] (mpfr_ptr, mpfr_srcptr, mp_rnd_t): EIF_INTEGER"
		end

	mpfr_init_set_ui (rop: POINTER; op, rnd: INTEGER): INTEGER is
			-- Initialize `rop' to the value `op' rounded in the direction `rnd'
			-- Default precision is used
			-- Returns: 0 if `rop' = `op'
			--          positive if `rop' > `op'
			--          negative if `rop' < `op'
		external
			"C [macro %"mpfr.h%"] (mpfr_ptr, unsigned long, mp_rnd_t): EIF_INTEGER"
		end

	mpfr_init_set_si (rop: POINTER; op, rnd: INTEGER): INTEGER is
			-- Initialize `rop' to the value `op' rounded in the direction `rnd'
			-- Default precision is used
			-- Returns: 0 if `rop' = `op'
			--          positive if `rop' > `op'
			--          negative if `rop' < `op'
		external
			"C [macro %"mpfr.h%"] (mpfr_ptr, long, mp_rnd_t): EIF_INTEGER"
		end

	mpfr_init_set_d (rop: POINTER; op: DOUBLE; rnd: INTEGER): INTEGER is
			-- Initialize `rop' to the value `op' rounded in the direction `rnd'
			-- Default precision is used
			-- Returns: 0 if `rop' = `op'
			--          positive if `rop' > `op'
			--          negative if `rop' < `op'
		external
			"C [macro %"mpfr.h%"] (mpfr_ptr, double, mp_rnd_t): EIF_INTEGER"
		end

	mpfr_init_set_f (rop, op: POINTER; rnd: INTEGER): INTEGER is
			-- Initialize `rop' to the value `op' rounded in the direction `rnd'
			-- Default precision is used
			-- Returns: 0 if `rop' = `op'
			--          positive if `rop' > `op'
			--          negative if `rop' < `op'
		external
			"C [macro %"mpfr.h%"] (mpfr_ptr, mpf_srcptr, mp_rnd_t): EIF_INTEGER"
		end

	mpfr_init_set_z (rop, op: POINTER; rnd: INTEGER): INTEGER is
			-- Initialize `rop' to the value `op' rounded in the direction `rnd'
			-- Default precision is used
			-- Returns: 0 if `rop' = `op'
			--          positive if `rop' > `op'
			--          negative if `rop' < `op'
		external
			"C [macro %"mpfr.h%"] (mpfr_ptr, mpz_srcptr, mp_rnd_t): EIF_INTEGER"
		end

	mpfr_init_set_q (rop, op: POINTER; rnd: INTEGER): INTEGER is
			-- Initialize `rop' to the value `op' rounded in the direction `rnd'
			-- Default precision is used
			-- Returns: 0 if `rop' = `op'
			--          positive if `rop' > `op'
			--          negative if `rop' < `op'
		external
			"C [macro %"mpfr.h%"] (mpfr_ptr, mpq_srcptr, mp_rnd_t): EIF_INTEGER"
		end

	mpfr_init_set_str (rop, s: POINTER; a_base, rnd: INTEGER): INTEGER is
				-- Initialize and set `rop' to the value represented by `s' in base `a_base'
				-- with rounding mode `rnd'
				-- Returns: -1 if an internal overflow occurs
				--           0 if no error occured
				--           1 if input is incorrect
		external
			"C (mpfr_ptr, __gmp_const char *, int, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

feature -- Conversion

	mpfr_get_d (op: POINTER; rnd: INTEGER): DOUBLE is
			-- Convert `op' to a double using rounding mode `rnd'
		external
			"C (mpfr_srcptr, mp_rnd_t): EIF_DOUBLE | %"mpfr.h%""
		end

	mpfr_get_d1 (op: POINTER): DOUBLE is
			-- Convert `op' to a double using the default rounding mode
		external
			"C (mpfr_srcptr): EIF_DOUBLE | %"mpfr.h%""
		end

	mpfr_get_z_exp (z, op: POINTER): INTEGER is
			-- Put the mantissa of `op' into `z' and return the exponent such
			-- that `op' = `z' * (2 ^ `z')
		external
			"C (mpz_ptr, mpfr_srcptr): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_get_str (str, expptr: POINTER; a_base, n_digits: INTEGER; op: POINTER; rnd: INTEGER): POINTER is
			-- Convert `op' to a string of digits in base `a_base' using rounding mode `rnd'
			-- Generate exactly `n_digits' significant digits
			-- If `n_digits' is 0 then generate the maximum possible number of digits
		external
			"C (char *, mp_exp_t *, int, size_t, mpfr_srcptr, mp_rnd_t): EIF_POINTER | %"mpfr.h%""
		end

feature -- Basic arithmetic

	mpfr_add (rop, op1, op2: POINTER; rnd: INTEGER): INTEGER is
			-- Set `rop' to `op1' + `op2' using rounding mode `rnd'
			-- Returns: 0 if `rop' = `op1' + `op2'
			--          positive if `rop' > `op1' + `op2'
			--          negative if `op1' + `op2'
		external
			"C (mpfr_ptr, mpfr_srcptr, mpfr_srcptr, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_add_ui (rop, op1: POINTER; op2, rnd: INTEGER): INTEGER is
		external
			"C (mpfr_ptr, mpfr_srcptr, unsigned long, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_add_z (rop, op1, op2: POINTER; rnd: INTEGER): INTEGER is
			-- Set `rop' to `op1' + `op2' using rounding mode `rnd'
			-- Returns: 0 if `rop' = `op1' + `op2'
			--          positive if `rop' > `op1' + `op2'
			--          negative if `op1' + `op2'
		external
			"C (mpfr_ptr, mpfr_srcptr, mpz_srcptr, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_add_q (rop, op1, op2: POINTER; rnd: INTEGER): INTEGER is
			-- Set `rop' to `op1' + `op2' using rounding mode `rnd'
			-- Returns: 0 if `rop' = `op1' + `op2'
			--          positive if `rop' > `op1' + `op2'
			--          negative if `op1' + `op2'
		external
			"C (mpfr_ptr, mpfr_srcptr, mpq_srcptr, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_sub (rop, op1, op2: POINTER; rnd: INTEGER): INTEGER is
			-- Set `rop' to `op1' - `op2' using rounding mode `rnd'
			-- Returns: 0 if `rop' = `op1' - `op2'
			--          positive if `rop' > `op1' - `op2'
			--          negative if `op1' - `op2'
		external
			"C (mpfr_ptr, mpfr_srcptr, mpfr_srcptr, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_ui_sub (rop: POINTER; op1: INTEGER; op2: POINTER; rnd: INTEGER): INTEGER is
			-- Set `rop' to `op1' - `op2' using rounding mode `rnd'
			-- Returns: 0 if `rop' = `op1' - `op2'
			--          positive if `rop' > `op1' - `op2'
			--          negative if `op1' - `op2'
		external
			"C (mpfr_ptr, unsigned long, mpfr_srcptr, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_sub_ui (rop, op1: POINTER; op2, rnd: INTEGER): INTEGER is
			-- Set `rop' to `op1' - `op2' using rounding mode `rnd'
			-- Returns: 0 if `rop' = `op1' - `op2'
			--          positive if `rop' > `op1' - `op2'
			--          negative if `op1' - `op2'
		external
			"C (mpfr_ptr, mpfr_srcptr, unsigned long, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_sub_z (rop, op1, op2: POINTER; rnd: INTEGER): INTEGER is
			-- Set `rop' to `op1' - `op2' using rounding mode `rnd'
			-- Returns: 0 if `rop' = `op1' - `op2'
			--          positive if `rop' > `op1' - `op2'
			--          negative if `op1' - `op2'
		external
			"C (mpfr_ptr, mpfr_srcptr, mpz_srcptr, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_sub_q (rop, op1, op2: POINTER; rnd: INTEGER): INTEGER is
			-- Set `rop' to `op1' - `op2' using rounding mode `rnd'
			-- Returns: 0 if `rop' = `op1' - `op2'
			--          positive if `rop' > `op1' - `op2'
			--          negative if `op1' - `op2'
		external
			"C (mpfr_ptr, mpfr_srcptr, mpq_srcptr, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_mul (rop, op1, op2: POINTER; rnd: INTEGER): INTEGER is
			-- Set `rop' to `op1' * `op2' using rounding mode `rnd'
			-- Returns: 0 if `rop' = `op1' * `op2'
			--          positive if `rop' > `op1' * `op2'
			--          negative if `op1' * `op2'
		external
			"C (mpfr_ptr, mpfr_srcptr, mpfr_srcptr, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_mul_ui (rop, op1: POINTER; op2, rnd: INTEGER): INTEGER is
			-- Set `rop' to `op1' * `op2' using rounding mode `rnd'
			-- Returns: 0 if `rop' = `op1' * `op2'
			--          positive if `rop' > `op1' * `op2'
			--          negative if `op1' * `op2'
		external
			"C (mpfr_ptr, mpfr_srcptr, unsigned long, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_mul_z (rop, op1, op2: POINTER; rnd: INTEGER): INTEGER is
			-- Set `rop' to `op1' * `op2' using rounding mode `rnd'
			-- Returns: 0 if `rop' = `op1' * `op2'
			--          positive if `rop' > `op1' * `op2'
			--          negative if `op1' * `op2'
		external
			"C (mpfr_ptr, mpfr_srcptr, mpz_srcptr, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_mul_q (rop, op1, op2: POINTER; rnd: INTEGER): INTEGER is
			-- Set `rop' to `op1' * `op2' using rounding mode `rnd'
			-- Returns: 0 if `rop' = `op1' * `op2'
			--          positive if `rop' > `op1' * `op2'
			--          negative if `op1' * `op2'
		external
			"C (mpfr_ptr, mpfr_srcptr, mpq_srcptr, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_div (rop, op1, op2: POINTER; rnd: INTEGER): INTEGER is
			-- Set `rop' to `op1' / `op2' using rounding mode `rnd'
			-- Returns: 0 if `rop' = `op1' / `op2'
			--          positive if `rop' > `op1' / `op2'
			--          negative if `op1' / `op2'
		external
			"C (mpfr_ptr, mpfr_srcptr, mpfr_srcptr, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_ui_div (rop: POINTER; op1: INTEGER; op2: POINTER; rnd: INTEGER): INTEGER is
			-- Set `rop' to `op1' / `op2' using rounding mode `rnd'
			-- Returns: 0 if `rop' = `op1' / `op2'
			--          positive if `rop' > `op1' / `op2'
			--          negative if `op1' / `op2'
		external
			"C (mpfr_ptr, unsigned long, mpfr_srcptr, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_div_ui (rop, op1: POINTER; op2, rnd: INTEGER): INTEGER is
			-- Set `rop' to `op1' / `op2' using rounding mode `rnd'
			-- Returns: 0 if `rop' = `op1' / `op2'
			--          positive if `rop' > `op1' / `op2'
			--          negative if `op1' / `op2'
		external
			"C (mpfr_ptr, mpfr_srcptr, unsigned long, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_div_z (rop, op1, op2: POINTER; rnd: INTEGER): INTEGER is
			-- Set `rop' to `op1' / `op2' using rounding mode `rnd'
			-- Returns: 0 if `rop' = `op1' / `op2'
			--          positive if `rop' > `op1' / `op2'
			--          negative if `op1' / `op2'
		external
			"C (mpfr_ptr, mpfr_srcptr, mpz_srcptr, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_div_q (rop, op1, op2: POINTER; rnd: INTEGER): INTEGER is
			-- Set `rop' to `op1' / `op2' using rounding mode `rnd'
			-- Returns: 0 if `rop' = `op1' / `op2'
			--          positive if `rop' > `op1' / `op2'
			--          negative if `op1' / `op2'
		external
			"C (mpfr_ptr, mpfr_srcptr, mpq_srcptr, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_sqrt (rop, op: POINTER; rnd: INTEGER): INTEGER is
			-- Set `rop' to the square root of `op' using rounding mode `rnd'
			-- Return 0 if result is exact and non-zero otherwise
		external
			"C (mpfr_ptr, mpfr_srcptr, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_sqrt_ui (rop: POINTER; op, rnd: INTEGER): INTEGER is
			-- Set `rop' to the square root of `op' using rounding mode `rnd'
			-- Return 0 if result is exact and non-zero otherwise
		external
			"C (mpfr_ptr, unsigned long, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_pow_ui (rop, op1: POINTER; op2, rnd: INTEGER): INTEGER is
			-- Set `rop' to `op1' raised to the power `op2' using rounding mode `rnd'
			-- Return 0 if result is exact and non-zero otherwise
		external
			"C (mpfr_ptr, mpfr_srcptr, unsigned long int, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_ui_pow_ui (rop: POINTER; op1, op2, rnd: INTEGER): INTEGER is
			-- Set `rop' to `op1' raised to the power `op2' using rounding mode `rnd'
			-- Return 0 if result is exact and non-zero otherwise
		external
			"C (mpfr_ptr, unsigned long int, unsigned long int, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_ui_pow (rop: POINTER; op1: INTEGER; op2: POINTER; rnd: INTEGER): INTEGER is
			-- Set `rop' to `op1' raised to the power `op2' using rounding mode `rnd'
			-- Return: 0 if result is exact
			--         positive if > `op1' ^ `op2'
			--         negative if < `op1' ^ `op2'
		external
			"C (mpfr_ptr, unsigned long int, mpfr_srcptr, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_pow_si (rop, op1: POINTER; op2, rnd: INTEGER): INTEGER is
			-- Set `rop' to `op1' raised to the power `op2' using rounding mode `rnd'
			-- Return 0 if result is exact and non-zero otherwise
		external
			"C (mpfr_ptr, mpfr_srcptr, long int, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_pow (rop, op1, op2: POINTER; rnd: INTEGER): INTEGER is	-- WAS A PROBLEM WHEN LINKING
			-- Set `rop' to `op1' raised to the power `op2' using rounding mode `rnd'
			-- Return 0 if result is exact and non-zero otherwise
		external
			"C (mpfr_ptr, mpfr_srcptr,mpfr_srcptr, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_neg (rop, op: POINTER; rnd: INTEGER): INTEGER is
			-- Set `rop' to -`op'
		external
			"C (mpfr_ptr, mpfr_srcptr, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_abs (rop, op: POINTER; rnd: INTEGER): INTEGER is
			-- Set `rop' to the absolute value of `op'
			-- Return: 0 if result is exact
			--         positive if > abs ('op')
			--         negative if < abs ('op')
		external
			"C [macro %"mpfr.h%"] (mpfr_ptr, mpfr_srcptr, mp_rnd_t): EIF_INTEGER"
		end

	mpfr_mul_2ui (rop, op1: POINTER; op2, rnd: INTEGER): INTEGER is
			-- Set `rop' to `op1' * (2 ^ `op2') using rounding mode `rnd'
			-- Return: 0 if `rop' = `op1'
			--         positive if `rop' > `op1'
			--         negative if `rop' < `op1'
		external
			"C (mpfr_ptr, mpfr_srcptr, unsigned long int, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_mul_2si (rop, op1: POINTER; op2, rnd: INTEGER): INTEGER is
			-- Set `rop' to `op1' * (2 ^ `op2') using rounding mode `rnd'
			-- Return: 0 if `rop' = `op1'
			--         positive if `rop' > `op1'
			--         negative if `rop' < `op1'
		external
			"C (mpfr_ptr, mpfr_srcptr, long int, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_div_2ui (rop, op1: POINTER; op2, rnd: INTEGER): INTEGER is
			-- Set `rop' to `op1' / (2 ^ `op2') using rounding mode `rnd'
			-- Return: 0 if `rop' = `op1'
			--         positive if `rop' > `op1'
			--         negative if `rop' < `op1'
		external
			"C (mpfr_ptr, mpfr_srcptr, unsigned long int, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_div_2si (rop, op1: POINTER; op2, rnd: INTEGER): INTEGER is
			-- Set `rop' to `op1' / (2 ^ `op2') using rounding mode `rnd'
			-- Return: 0 if `rop' = `op1'
			--         positive if `rop' > `op1'
			--         negative if `rop' < `op1'
		external
			"C (mpfr_ptr, mpfr_srcptr, long int, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

feature -- Comparison

	mpfr_cmp (op1, op2: POINTER): INTEGER is
			-- Compare `op1' and `op2'
			-- Return: 0 if `op1' = `op2'
			--         positive if `op1' > `op2'
			--         negative if `op1' < `op2'
		external
			"C [macro %"mpfr.h%"] (mpfr_srcptr, mpfr_srcptr): EIF_INTEGER"
		end

	mpfr_cmp_ui (op1: POINTER; op2: INTEGER): INTEGER is
			-- Compare `op1' and `op2'
			-- Return: 0 if `op1' = `op2'
			--         positive if `op1' > `op2'
			--         negative if `op1' < `op2'
		external
			"C [macro %"mpfr.h%"] (mpfr_srcptr, unsigned long int): EIF_INTEGER"
		end

	mpfr_cmp_si (op1: POINTER; op2: INTEGER): INTEGER is
			-- Compare `op1' and `op2'
			-- Return: 0 if `op1' = `op2'
			--         positive if `op1' > `op2'
			--         negative if `op1' < `op2'
		external
			"C [macro %"mpfr.h%"] (mpfr_srcptr, signed long int): EIF_INTEGER"
		end

	mpfr_cmp_ui_2exp (op1: POINTER; op2, e: INTEGER): INTEGER is
			-- Compare `op1' and `op2' multiplied by two to the power e
		external
			"C (mpfr_srcptr, unsigned long int, int): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_cmp_si_2exp (op1: POINTER; op2, e: INTEGER): INTEGER is
			-- Compare `op1' and `op2' multiplied by two to the power e
		external
			"C (mpfr_srcptr, long int, int): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_eq (op1, op2: POINTER; op3: INTEGER): INTEGER is
			-- Return non-zero if the first `op3' bits of `op1' and `op2'
			-- are equal and zero otherwise
		external
			"C (mpfr_srcptr, mpfr_srcptr, unsigned long): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_nan_p (op: POINTER): INTEGER is
			-- Return non-zero if `op' is NaN and zero otherwise
		external
			"C (mpfr_srcptr): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_inf_p (op: POINTER): INTEGER is
			-- Return non-zero if `op' is +/- infinity and zero otherwise
		external
			"C (mpfr_srcptr): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_number_p (op: POINTER): INTEGER is
			-- Return non-zero if `op' is an ordinary number
			-- ie not NaN or infinity
		external
			"C (mpfr_srcptr): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_reldiff (rop, op1, op2: POINTER; rnd: INTEGER) is
			-- Set `rop' to the relative difference between `op1' and `op2'
		external
			"C (mpfr_ptr, mpfr_srcptr, mpfr_srcptr, mp_rnd_t) | %"mpfr.h%""
		end

	mpfr_sgn (op: POINTER): INTEGER is
				-- negative if `op' < 0
				-- 0 if `op' = 0
				-- positive if `op' > 0
		external
			"C [macro %"mpfr.h%"] (mpfr_srcptr): EIF_INTEGER"
		end

feature -- Special

	mpfr_log (rop, op: POINTER; rnd: INTEGER): INTEGER is
			-- Set `rop' to the natural logarithm of `op' using rounding mode `rnd'
			-- Return 0 if the result is exact and non-zero otherwise
		external
			"C (mpfr_ptr, mpfr_srcptr, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_exp (rop, op: POINTER; rnd: INTEGER): INTEGER is
			-- Set `rop' to the exponential of `op' using rounding mode `rnd'
			-- Return: 0 if `rop' = exp (`op')
			--         positive if `rop' > exp (`op')
			--         negative if `rop' < exp (`op')
		external
			"C (mpfr_ptr, mpfr_srcptr, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_exp2 (rop, op: POINTER; rnd: INTEGER): INTEGER is
			-- Set `rop' to the exponential of `op' using rounding mode `rnd'
			-- Return: 0 if `rop' = exp (`op')
			--         positive if `rop' > exp (`op')
			--         negative if `rop' < exp (`op')
		external
			"C (mpfr_ptr, mpfr_srcptr, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_cos (cop, op: POINTER; rnd: INTEGER): INTEGER is
			-- Set `cop' to the cosine of `op' using rounding mode `rnd'
			-- Return 0 if result is exact and non-zero otherwise
		external
			"C (mpfr_ptr, mpfr_srcptr, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_sin (sop, op: POINTER; rnd: INTEGER): INTEGER is
			-- Set `sop' to the sine of `op' using rounding mode `rnd'
			-- Return 0 if result is exact and non-zero otherwise
		external
			"C (mpfr_ptr, mpfr_srcptr, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_tan (top, op: POINTER; rnd: INTEGER): INTEGER is
			-- Set `top' to the tangent of `op' using rounding mode `rnd'
			-- Return 0 if result is exact and non-zero otherwise
		external
			"C (mpfr_ptr, mpfr_srcptr, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_sin_cos (sop, cop, op: POINTER; rnd: INTEGER): INTEGER is
			-- Set `sop' to the sine of `op' using rounding mode `rnd'
			-- Set `cop' to the cosine of `op' using rounding mode `rnd'
			-- Return 0 iff both results are exact
		external
			"C (mpfr_ptr, mpfr_ptr, mpfr_srcptr, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_acos (rop, op: POINTER; rnd: INTEGER): INTEGER is
			-- Set `rop' to the arc-cosine of `op' using rounding mode `rnd'
			-- Return 0 iff result is exact
		external
			"C (mpfr_ptr, mpfr_srcptr, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_asin (rop, op: POINTER; rnd: INTEGER): INTEGER is
			-- Set `rop' to the arc-sine of `op' using rounding mode `rnd'
			-- Return 0 iff result is exact
		external
			"C (mpfr_ptr, mpfr_srcptr, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_atan (rop, op: POINTER; rnd: INTEGER): INTEGER is
			-- Set `rop' to the arc-tangent of `op' using rounding mode `rnd'
			-- Return 0 iff result is exact
		external
			"C (mpfr_ptr, mpfr_srcptr, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_cosh (cop, op: POINTER; rnd: INTEGER): INTEGER is
			-- Set `cop' to the hyperbolic cosine of `op' using rounding mode `rnd'
			-- Return 0 iff result is exact
		external
			"C (mpfr_ptr, mpfr_srcptr, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_sinh (sop, op: POINTER; rnd: INTEGER): INTEGER is
			-- Set `sop' to the hyperbolic sine of `op' using rounding mode `rnd'
			-- Return 0 iff result is exact
		external
			"C (mpfr_ptr, mpfr_srcptr, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_tanh (top, op: POINTER; rnd: INTEGER): INTEGER is
			-- Set `top' to the hyperbolic tangent of `op' using rounding mode `rnd'
			-- Return 0 iff result is exact
		external
			"C (mpfr_ptr, mpfr_srcptr, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_acosh (rop, op: POINTER; rnd: INTEGER): INTEGER is
			-- Set `rop' to the inverse hyperbolic cosine of `op' using rounding mode `rnd'
			-- Return 0 iff result is exact
		external
			"C (mpfr_ptr, mpfr_srcptr, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_asinh (rop, op: POINTER; rnd: INTEGER): INTEGER is
			-- Set `rop' to the inverse hyperbolic sine of `op' using rounding mode `rnd'
			-- Return 0 iff result is exact
		external
			"C (mpfr_ptr, mpfr_srcptr, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_atanh (rop, op: POINTER; rnd: INTEGER): INTEGER is
			-- Set `rop' to the inverse hyperbolic tangent of `op' using rounding mode `rnd'
			-- Return 0 iff result is exact
		external
			"C (mpfr_ptr, mpfr_srcptr, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_fac_ui (rop: POINTER; op, rnd: INTEGER): INTEGER is
			-- Set `rop' to the factorial of `op' using rounding mode `rnd'
			-- Return 0 iff result is exact
		external
			"C (mpfr_ptr, unsigned long int, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_log1p (rop, op: POINTER; rnd: INTEGER): INTEGER is
			-- Set `rop' to the logarithm of one plus `op' using rounding mode `rnd'
			-- Return 0 iff result is exact
		external
			"C (mpfr_ptr, mpfr_srcptr, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_expm1 (rop, op: POINTER; rnd: INTEGER): INTEGER is
			-- Set `rop' to the exponential of `op' minus one using rounding mode `rnd'
			-- Return 0 iff result is exact
		external
			"C (mpfr_ptr, unsigned long int, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_log2 (rop, op: POINTER; rnd: INTEGER): INTEGER is
			-- Set `rop' to the base 2 logarithm of `op' using rounding mode `rnd'
			-- Return 0 iff result is exact
		external
			"C (mpfr_ptr, unsigned long int, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_log10 (rop, op: POINTER; rnd: INTEGER): INTEGER is
			-- Set `rop' to the base 10 logarithm of `op' using rounding mode `rnd'
			-- Return 0 iff result is exact
		external
			"C (mpfr_ptr, unsigned long int, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_fma (rop, opx, opy, opz: POINTER; rnd: INTEGER): INTEGER is
			-- Set `rop' to `opx' * `opy' + `opz' using rounding mode `rnd'
			-- Return: 0 if `rop' = `opx' * `opy' + `opz'
			--         positive if `rop' > `opx' * `opy' + `opz'
			--         negative if `rop' < `opx' * `opy' + `opz'
		external
			"C (mpfr_ptr, mpfr_srcptr, mpfr_srcptr, mpfr_srcptr, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_agm (rop, op1, op2: POINTER; rnd: INTEGER) is
			-- Set `rop' to the arithmetic-geometric mean of `op1' and `op2'
			-- using rounding mode `rnd'
		external
			"C (mpfr_ptr, mpfr_srcptr, mpfr_srcptr, mp_rnd_t) | %"mpfr.h%""
		end

	mpfr_const_log2 (rop: POINTER; rnd: INTEGER) is
			-- Set `rop' to the logarithm of 2 using rounding mode `rnd'
		external
			"C (mpfr_ptr, mp_rnd_t) | %"mpfr.h%""
		end

	mpfr_const_pi (rop: POINTER; rnd: INTEGER) is
			-- Set `rop' to Pi using rounding mode `rnd'
		external
			"C (mpfr_ptr, mp_rnd_t) | %"mpfr.h%""
		end

	mpfr_const_euler (rop: POINTER; rnd: INTEGER) is
			-- Set `rop' to Euler's constant (0.577...) using rounding mode `rnd'
		external
			"C (mpfr_ptr, mp_rnd_t) | %"mpfr.h%""
		end

feature -- Miscellaneous

	mpfr_rint (rop, op: POINTER; rnd: INTEGER): INTEGER is
			-- Set `rop' to `op' rounded to an integer using rounding mode `rnd'
		external
			"C (mpfr_ptr, mpfr_srcptr, mp_rnd_t): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_ceil (rop, op: POINTER): INTEGER is
			-- Set `rop' to `op' rounded to the next higher integer
		external
			"C (mpfr_ptr, mpfr_srcptr): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_floor (rop, op: POINTER): INTEGER is
			-- Set `rop' to `op' rounded to the next lower integer
		external
			"C (mpfr_ptr, mpfr_srcptr): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_round (rop, op: POINTER): INTEGER is
			-- Set `rop' to `op' rounded to the nearest representable integer
		external
			"C (mpfr_ptr, mpfr_srcptr): EIF_INTEGER | %"mpfr.h%""
		end

	mpfr_trunc (rop, op: POINTER): INTEGER is
			-- Set `rop' to `op' rounded towards zero
		external
			"C (mpfr_ptr, mpfr_srcptr): EIF_INTEGER | %"mpfr.h%""
		end

end -- class MPFR_FUNCTIONS
