indexing
	description: "Complex numbers"
	author: "Christopher John Saunders"
	date: "$Date: 2003/10/17 $"
	revision: "$Revision: 1.0 $"

deferred class
	COMPLEX [G -> NUMERIC create default_create end]

inherit
	NUMERIC
		redefine
			copy, is_equal, out
		end

feature -- Access

	conjugate: like Current is
			-- Conjugate
		deferred
		ensure
			result_exists: Result /= Void
		end

	i: like Current is
			-- Square root of -1
		deferred
		ensure
			result_exists: Result /= Void
			is_i: Result.real.is_equal (real.zero) and Result.imaginary.is_equal (imaginary.one)
		end

	imaginary: like real
			-- Imaginary part

	one: like Current is
			-- Neutral element for "*" and "/"
		deferred
		ensure then
			result_exists: Result /= Void
			is_one: Result.real.is_equal (real.one) and Result.imaginary.is_equal (imaginary.zero)
		end

	real: G --is	-- Real part

	zero: like Current is
			-- Neutral element for "+" and "-"
		deferred
		ensure then
			result_exists: Result /= Void
			is_zero: Result.real.is_equal (real.zero) and Result.imaginary.is_equal (imaginary.zero)
		end

feature -- Measurement

	norm: like real is
			-- Normal
		do
			Result := (real * real) + (imaginary * imaginary)
		end

feature -- Comparison

	is_equal (other: like Current): BOOLEAN is
			-- Is `other' attached to an object considered
			-- equal to current object?
		do
			Result := real.is_equal (other.real) and then imaginary.is_equal (other.imaginary)
		end

feature -- Status report

	divisible (other: like Current): BOOLEAN is
			-- May current object be divided by `other'?
		do
			Result := not (other.real.is_equal (real.zero) and then other.imaginary.is_equal (imaginary.zero))
		end

feature -- Element change

	set (re, im: like real) is
			-- Set `real' to `re' and `imaginary' to `im'
		require
			re_exists: re /= Void
			im_exists: im /= Void
		deferred
		ensure
			real_set: real.is_equal (re)
			imaginary_set: imaginary.is_equal (im)
		end

	set_imaginary (im: like real) is
			-- Set `imaginary' to `im'
		require
			im_exists: im /= Void
		deferred
		ensure
			imaginary_set: imaginary.is_equal (im)
		end

	set_real (re: like real) is
			-- Set `real' to `re'
		require
			re_exists: re /= Void
		deferred
		ensure
			real_set: real.is_equal (re)
		end

feature -- Duplication

	copy (other: like Current) is
			-- Update current object using fields of object attached
			-- to `other', so as to yield equal objects.
		do
			set (other.real, other.imaginary)
		end

feature -- Basic operations

	infix "^" (other: NUMERIC): NUMERIC is
			-- Raise to the power `other'
		require
			other_exists: other /= Void
			good_exponent: exponentiable (other)
		deferred
		ensure
			result_exists: Result /= Void
		end
		
feature -- Output

	out: STRING is
			-- Simple string representation
		do
			Result := "(" + real.out + ", " + imaginary.out + ")"
		end

invariant
	real_exists: real /= Void
	imaginary_exists: imaginary /= Void

end -- class COMPLEX
