indexing
	description: "Objects that ..."
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	GMP_INTEGER_COMPLEX_REF

inherit
	COMPLEX [GMP_INTEGER_REF]
		redefine
			default_create
		end

create
	default_create, make, make_from_integer_reference

feature {NONE} -- Initialization

	default_create is
			-- Initialize `Current'.
		do
			create real
			create imaginary
		end

	make (re, im: like real) is
			-- Set `real' from `re' and `imaginary' from `im'
		do
			create real.make_from_gmp_integer (re)
			create imaginary.make_from_gmp_integer (im)
		end

	make_from_integer_reference (re, im: like real) is
			-- Set `real' from `re' and `imaginary' from `im'
		do
			real := re
			imaginary := im
		ensure
			real_set: real = re
			imaginary_set: imaginary = im
		end

feature -- Access

	conjugate: like Current is
			-- Conjugate
		do
			create Result.make_from_integer_reference (real, -imaginary)
		end

	i: like Current is
			-- Square root of -1
		do
			create Result.make_from_integer_reference (real.zero, imaginary.one)
		end

	one: like Current is
			-- Neutral element for "*" and "/"
		do
			create Result.make_from_integer_reference (real.one, imaginary.zero)
		end

	zero: like Current is
			-- Neutral element for "+" and "-"
		do
			create Result.make_from_integer_reference (real.zero, imaginary.zero)
		end

feature -- Status report

	exponentiable (other: NUMERIC): BOOLEAN is
			-- May current object be elevated to the power `other'?
		do
		end

feature -- Element change

	set (re, im: like real) is
			-- Set `real' to `re' and `imaginary' to `im'
		do
			real.copy (re)
			imaginary.copy (im)
		end

	set_imaginary (im: like real) is
			-- Set `imaginary' to `im'
		do
			imaginary.copy (im)
		end

	set_real (re: like real) is
			-- Set `real' to `re'
		do
			real.copy (re)
		end

feature -- Basic operations

	infix "+" (other: like Current): like Current is
			-- Sum with `other' (commutative).
		do
			create Result.make_from_integer_reference (real + other.real, imaginary + other.imaginary)
		end

	infix "-" (other: like Current): like Current is
			-- Result of subtracting `other'
		do
			create Result.make_from_integer_reference (real - other.real, imaginary - other.imaginary)
		end

	infix "*" (other: like Current): like Current is
			-- Product by `other'
		do
			create Result.make_from_integer_reference (real * other.real - imaginary * other.imaginary,
			                                           real * other.imaginary + imaginary * other.real)
		end

	infix "/" (other: like Current): like Current is
			-- Division by `other'
		local
			den: like real
		do
			den := other.real * other.real + other.imaginary * other.imaginary
			create Result.make_from_integer_reference ((real * other.real + imaginary * other.imaginary) // den,
			                                           (imaginary * other.real - real * other.imaginary) // den)
		end

	prefix "+": like Current is
			-- Unary plus
		do
			create Result.make (real, imaginary)
		end

	prefix "-": like Current is
			-- Unary minus
		do
			create Result.make_from_integer_reference (-real, -imaginary)
		end

feature {NONE} -- Inapplicable

	infix "^" (other: NUMERIC): NUMERIC is
			-- Raise to the power `other'
		do
		end

end -- class GMP_INTEGER_COMPLEX_REF
