indexing
	description: "Pseudo-random number sequence of MPFR_REFs"
	author: "Christopher John Saunders"
	date: "$Date: 2003/10/17 $"
	revision: "$Revision: 1.0 $"

class
	GMP_RANDOM_MPFR

inherit
	COUNTABLE_SEQUENCE [MPFR_REF]
		redefine
			default_create, start
		end

	GMP_RANDOM_MPFR_FUNCTIONS
		export
			{NONE} all
		redefine
			default_create
		end

create
	default_create, make

feature {NONE} -- Initialization

	default_create is
			-- Initialize `Current'.
		do
			compare_objects
			create last_i_th
		end

	make (prec: INTEGER) is
			-- Set the precision of `item' to `prec'
		require
			prec_in_range: (feature {MPFR_PRECISION_CONSTANTS}.Mpfr_prec_min <= prec) and
			               (prec <= feature {MPFR_PRECISION_CONSTANTS}.Mpfr_prec_max)
		do
			compare_objects
			create last_i_th
			last_i_th.set_precision (prec)
		ensure
			last_i_th_precision_set: last_i_th.precision = prec
		end
		
feature -- Access

	i_th (i: INTEGER): like item is
			-- The `i'-th item
		local
			n: INTEGER
		do
			if last_i < i then
					-- No need to restart
				from
					n := last_i + 1
				until
					n > i
				loop
					mpfr_urandomb (last_i_th.item, state.item)
					n := n + 1
				end
				create Result.make_from_mpfr (last_i_th)
				last_i := n - 1
			elseif i < last_i then
					-- Need to restart
				from
					start
					n := 2
				until
					n = i
				loop
					mpfr_urandomb (last_i_th.item, state.item)
					n := n + 1
				end
				create Result.make_from_mpfr (last_i_th)
				last_i := n
			else
					-- Result is `last_i_th'
				create Result.make_from_mpfr (last_i_th)
			end
		end

feature -- Cursor movement

	start is
			-- Move to first position.
		do
			index := 1
			create state
			last_i := 1
			mpfr_urandomb (last_i_th.item, state.item)
		end

feature {NONE} -- Implementation

	last_i: INTEGER

	last_i_th: like item

	state: GMP_RANDSTATE_T

end -- class GMP_RANDOM_MPFR
