indexing
	description: "Pseudo-random number sequence of GMP_RATIONAL_REFs"
	author: "Christopher John Saunders"
	date: "$Date: 2003/10/17 $"
	revision: "$Revision: 1.0 $"

class
	GMP_RANDOM_RATIONAL

inherit
	COUNTABLE_SEQUENCE [GMP_RATIONAL_REF]
		redefine
			start
		end

	GMP_RANDOM_INTEGER_FUNCTIONS
		export
			{NONE} all
		end

create
	make

feature {NONE} -- Initialization

	make (n_max, d_max: GMP_INTEGER_REF) is
			-- Initialize `Current'.
		do
			compare_objects
			create last_i_th_numerator
			create last_i_th_denominator
			create numerator_maximum_value.make_from_gmp_integer (n_max)
			create denominator_maximum_value.make_from_gmp_integer (d_max)
		end

feature -- Access

	i_th (i: INTEGER): like item is
			-- The `i'-th item
		local
			n: INTEGER
		do
			if last_i < i then
					-- No need to restart
				from
					n := last_i + 1
				until
					n > i
				loop
					mpz_urandomm (last_i_th_numerator.item, state.item, numerator_maximum_value.item)
					from
						mpz_urandomm (last_i_th_denominator.item, state.item, denominator_maximum_value.item)
					until
						not last_i_th_denominator.is_equal (zero)
					loop
						mpz_urandomm (last_i_th_denominator.item, state.item, denominator_maximum_value.item)
					end
					n := n + 1
				end
				create Result.make (last_i_th_numerator, last_i_th_denominator)
				last_i := n - 1
			elseif i < last_i then
					-- Need to restart
				from
					start
					n := 2
				until
					n = i
				loop
					mpz_urandomm (last_i_th_numerator.item, state.item, numerator_maximum_value.item)
					from
						mpz_urandomm (last_i_th_denominator.item, state.item, denominator_maximum_value.item)
					until
						not last_i_th_denominator.is_equal (zero)
					loop
						mpz_urandomm (last_i_th_denominator.item, state.item, denominator_maximum_value.item)
					end
					n := n + 1
				end
				create Result.make (last_i_th_numerator, last_i_th_denominator)
				last_i := n
			else
					-- Result is `last_i_th'
				create Result.make (last_i_th_numerator, last_i_th_denominator)
			end
		end

	numerator_maximum_value: GMP_INTEGER_REF
			-- The largest numerator generated will be
			-- `maximum_value' - 1

	denominator_maximum_value: GMP_INTEGER_REF
			-- The largest denominato generated will be
			-- `maximum_value' - 1

feature -- Cursor movement

	start is
			-- Move to first position.
		do
			index := 1
			create state
			last_i := 1
			mpz_urandomm (last_i_th_numerator.item, state.item, numerator_maximum_value.item)
			mpz_urandomm (last_i_th_denominator.item, state.item, denominator_maximum_value.item)
		end

feature {NONE} -- Implementation

	last_i: INTEGER

	last_i_th_numerator: GMP_INTEGER_REF

	last_i_th_denominator: GMP_INTEGER_REF

	state: GMP_RANDSTATE_T

	zero: GMP_INTEGER_REF is
			-- Zero
		once
			create Result
		end
		
end -- class GMP_RANDOM_RATIONAL
