indexing
	description: "Pseudo-random number sequence of GMP_REAL_REFs"
	author: "Christopher John Saunders"
	date: "$Date: 2003/10/17 $"
	revision: "$Revision: 1.0 $"

class
	GMP_RANDOM_REAL

inherit
	COUNTABLE_SEQUENCE [GMP_REAL_REF]
		redefine
			start
		end

	GMP_RANDOM_REAL_FUNCTIONS
		export
			{NONE} all
		end

create
	make

feature {NONE} -- Initialization

	make (a_precision: INTEGER) is
			-- Initialize `Current'.
		do
			compare_objects
			create last_i_th
			precision := a_precision
		end

feature -- Access

	i_th (i: INTEGER): like item is
			-- The `i'-th item
		local
			n: INTEGER
		do
			if last_i < i then
					-- No need to restart
				from
					n := last_i + 1
				until
					n > i
				loop
					mpf_urandomb (last_i_th.item, state.item, precision)
					n := n + 1
				end
				create Result.make_from_gmp_real (last_i_th)
				last_i := n - 1
			elseif i < last_i then
					-- Need to restart
				from
					start
					n := 2
				until
					n = i
				loop
					mpf_urandomb (last_i_th.item, state.item, precision)
					n := n + 1
				end
				create Result.make_from_gmp_real (last_i_th)
				last_i := n
			else
					-- Result is `last_i_th'
				create Result.make_from_gmp_real (last_i_th)
			end
		end

	precision: INTEGER
			-- The largest number generated will be
			-- `maximum_value' - 1

feature -- Cursor movement

	start is
			-- Move to first position.
		do
			index := 1
			create state
			last_i := 1
			mpf_urandomb (last_i_th.item, state.item, precision)
		end

feature {NONE} -- Implementation

	last_i: INTEGER

	last_i_th: like item

	state: GMP_RANDSTATE_T

end -- class GMP_RANDOM_REAL
