indexing
	description: "Objects that ..."
	author: "Christopher John Saunders"
	date: "$Date: 2003/10/17 $"
	revision: "$Revision: 1.0 $"

class
	GMP_RANDSTATE_T

inherit
	MEMORY
		export
			{NONE} all
			{ANY} dispose
		redefine
			default_create, dispose, out
		end

	MEMORY_STRUCTURE
		rename
			make as memory_structure_make
		export
			{NONE} all
			{ANY} item
		redefine
			default_create, out
		end

create
	default_create, make, make_size

feature {NONE} -- Initialization

	default_create is
			-- Initialize `Current'.
		do
			memory_structure_make
			gmp_randinit_default (item)
		end

	make (a: GMP_INTEGER_REF; c, m2exp: INTEGER) is
			-- Initialize `Current' with a linear congruential algorithm
			-- X = (`a' * X + `c') mod 2 ^ `m2exp'.
		require
			a_exists: a /= Void
		do
			memory_structure_make
			gmp_randinit_lc_2exp (item, a.item, c, m2exp)
		end

	make_size (bits: INTEGER) is
			-- Initialize `Current'.
		require
			bits_positive: bits > 0
			bits_small_enough: bits <= Maximum_bits
		do
			memory_structure_make
			gmp_randinit_lc_2exp_size (item, bits)
		end

feature -- Access

	algorithm: GMP_RANDALG_T is
			-- Algorithm used
		do
			create Result.make (mp_alg (item))
		end

	algorithm_data: GMP_RANDATA_LC is
			-- Algorithm specific data
		do
			create Result.make_by_pointer (mp_lc (item))
		end

	seed: GMP_INTEGER_REF is
			-- Seed
		do
			create Result.make_by_pointer (mp_seed (item))
		end

feature -- Measurement

	Maximum_bits: INTEGER is 128

	structure_size: INTEGER is
			-- Size to allocate (in bytes)
		do
			Result := c_structure_size
		end

feature -- Removal

	dispose is
			-- Action to be executed just before garbage collection
			-- reclaims an object.
		do
			gmp_randclear (item)
		end

feature -- Output

	out: STRING is
			-- Simple string representation
		do
			Result := "[algorithm_data: "
			Result.append_string (algorithm_data.out)
			Result.append_string (", seed: ")
			Result.append_string (seed.out)
			Result.append_character (']')
		end

feature {NONE} -- Externals

	c_structure_size: INTEGER is
			-- Size of  "C" struct __gmp_randstate_struct.
		external
			"C [macro %"gmp.h%"]"
		alias
			"sizeof (__gmp_randstate_struct)"
		end

	mp_seed (p: POINTER): POINTER is
		external
			"C [struct %"gmp.h%"] (__gmp_randstate_struct): EIF_POINTER"
		alias
			"_mp_seed"
		end

	mp_alg (p: POINTER): INTEGER is
		external
			"C [struct %"gmp.h%"] (__gmp_randstate_struct): EIF_INTEGER"
		alias
			"_mp_alg"
		end

	mp_lc (p: POINTER): POINTER is
		external
			"C [struct %"gmp.h%"] (__gmp_randstate_struct): EIF_POINTER"
		alias
			"_mp_algdata._mp_lc"
		end

	gmp_randinit_default (p: POINTER) is
		external
			"C (__gmp_randstate_struct *) | %"gmp.h%""
		end

	gmp_randinit_lc_2exp (p, a: POINTER; c, m2exp: INTEGER) is
		external
			"C (__gmp_randstate_struct *, mpz_srcptr, unsigned long int, unsigned long int) | %"gmp.h%""
		end

	gmp_randinit_lc_2exp_size (p: POINTER; size: INTEGER) is
		external
			"C (__gmp_randstate_struct *, unsigned long) | %"gmp.h%""
		end

	gmp_randclear (p: POINTER) is
		external
			"C (__gmp_randstate_struct *) | %"gmp.h%""
		end

end -- class GMP_RANDSTATE_T
