indexing
	description: "Prime number sequence of GMP_INTEGER_REFs"
	author: "Christopher John Saunders"
	date: "$Date: 2003/10/17 $"
	revision: "$Revision: 1.0 $"

class
	GMP_PRIMES

inherit
	COUNTABLE_SEQUENCE [GMP_INTEGER_REF]
		redefine
			changeable_comparison_criterion, default_create
		end

	GMP_INTEGER_MATH
		rename
			remove as gmp_integer_math_remove
		export
			{NONE} all
		redefine
			default_create
		end

create
	default_create

feature {NONE} -- Initialization

	default_create is
			-- Initialize `Current'.
			-- Use `is_equal' to compare objects.
		do
			changeable_comparison_criterion := True
			compare_objects
			changeable_comparison_criterion := False
			create last_i_th
		ensure then
			last_i_th_exists: last_i_th /= Void
			not_changeable_comparison_criterion: not changeable_comparison_criterion
		end

feature -- Access

	i_th (i: INTEGER): GMP_INTEGER_REF is
			-- The `i'-th item
		local
			n: INTEGER
		do
			if i = 1 then
					-- First prime is 2
				last_i_th.set_from_integer (2)
				Result := last_i_th
				last_i := 1
			elseif last_i < i then
					-- No need to restart
				from
					n := last_i + 1
				until
					n > i
				loop
					last_i_th := next_prime (last_i_th)
					n := n + 1
				end
				Result := last_i_th
				last_i := n - 1
			elseif i < last_i then
					-- Need to restart
				from
					last_i_th.set_from_integer (2)
					n := 2
				until
					n > i
				loop
					last_i_th := next_prime (last_i_th)
					n := n + 1
				end
				Result := last_i_th
				last_i := n - 1
			else
					-- Result is `last_i_th'
				Result := last_i_th
			end
		end

feature -- Status report

	changeable_comparison_criterion: BOOLEAN
			-- May `object_comparison' be changed?

feature {NONE} -- Implementation

	last_i: INTEGER

	last_i_th: like item

end -- class GMP_PRIMES
